import createAuth0Client from '@auth0/auth0-spa-js';
import SubmitService from '@/services/submit.js'
import DeleteService from '@/services/del.js'

export const namespaced = true;

export const state = {
    auth0Client: null,
    isLoading: true,
    isAuthenticated: false,
    popupOpen: false,
    user: null,
    error: null,
};

export const mutations = {
    SET_AUTH_INSTANCE(state, payload) {
        state.auth0Client = payload;
    },

    SET_ERROR(state, payload) {
        state.error = payload;
    },

    SET_LOADING(state, payload) {
        state.isLoading = payload;
    },

    SET_AUTHENTICATED(state, payload) {
        state.isAuthenticated = payload;
    },

    SET_USER(state, payload) {
        state.user = payload;
    },

    SET_POPUP(state, payload) {
        state.popupOpen = payload;
    },
};

export const actions = {
    async createAuthInstance({ commit, dispatch }, pluginOptions) {
        let auth;
       
        try {
        
            auth = await createAuth0Client({
                ...pluginOptions,
                domain: pluginOptions.domain,
                client_id: pluginOptions.clientId,
                audience: pluginOptions.audience,
                redirect_uri: pluginOptions.redirectUri,
            });
            
            commit('SET_AUTH_INSTANCE', auth);

            if (auth) {
                SubmitService.createInstance(await dispatch('getTokenSilently'));
                DeleteService.createInstance(await dispatch('getTokenSilently'));
            }

            if (
                window.location.search.includes('code=') &&
                window.location.search.includes('state=')
            ) {
                const { appState } = await auth.handleRedirectCallback();

                dispatch('onRedirectCallback', appState);
            }
        } catch (error) {
            commit('SET_ERROR', error);
        } finally {
            const isAuthenticated = await auth.isAuthenticated();
            const user = await auth.getUser();

            commit('SET_AUTHENTICATED', isAuthenticated);
            commit('SET_USER', user);
            console.log(user);
            commit('SET_LOADING', false);
        }
    },

    async loginWithPopup({ state, commit, dispatch }) {
        commit('SET_POPUP', true);

        try {
            await state.auth0Client.loginWithPopup(
                state.auth0Client.options,
                state.auth0Client.config
            );

            const isAuthenticated = await state.auth0Client.isAuthenticated();
            const user = await state.auth0Client.getUser();

            localStorage.setItem('access_token', await dispatch('getTokenSilently'));
            SubmitService.createInstance(await dispatch('getTokenSilently'));
            DeleteService.createInstance(await dispatch('getTokenSilently'));

            commit('SET_AUTHENTICATED', isAuthenticated);
            console.log(user)
            commit('SET_USER', user);
            commit('SET_LOADING', false);
        } catch (error) {
            commit('SET_ERROR', error);
            console.error(error);
        } finally {
            commit('SET_POPUP', false);
        }

        const user = await state.auth0Client.getUser();
        commit('SET_USER', user);
        commit('SET_AUTHENTICATED', true);
    },

    async handleRedirectCallback({ state }) {
        commit('SET_LOADING', true);

        try {
            await state.auth0Client.handleRedirectCallback();
            const user = await state.auth0Client.getUser();

            commit('SET_AUTHENTICATED', true);
            commit('SET_USER', user);
        } catch (error) {
            commit('SET_ERROR', error);
        } finally {
            commit('SET_LOADING', false);
        }
    },

    onRedirectCallback() {
        window.history.replaceState(
            {},
            document.title,
            window.location.pathname
        );
    },

    loginWithRedirect({ state }, o) {
        return state.auth0Client.loginWithRedirect(o);
    },
    /** Returns all the claims present in the ID token */
    getIdTokenClaims(o) {
        return state.auth0Client.getIdTokenClaims(o);
    },
    /** Returns the access token. If the token is invalid or missing, a new one is retrieved */
    async getTokenSilently(o) {
        return await state.auth0Client.getTokenSilently(o);
    },
    /** Gets the access token using a popup window */

    getTokenWithPopup(o) {
        return state.auth0Client.getTokenWithPopup(o);
    },
    /** Logs the user out and removes their session on the authorization server */
    async logout({ state, commit }) {
        try {
            await state.auth0Client.logout();
            localStorage.removeItem('access_token');
            commit('SET_AUTHENTICATED', false);
        } catch (error) {
            commit('SET_ERROR', error);
            console.error(error);
        }
    },
};

export const getters = {
    isAuthenticated: (state) => state.isAuthenticated,
    isLoading: (state) => state.isLoading,
    user: (state) => state.user,
};
